"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es6.object.define-property");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handle = handle;
exports.getResourcesHandler = getResourcesHandler;
exports.missingKeyHandler = void 0;

require("core-js/modules/es6.array.for-each");

require("core-js/modules/es6.regexp.split");

require("core-js/modules/es6.function.bind");

require("core-js/modules/es6.array.index-of");

var _lodash = _interopRequireDefault(require("lodash"));

var _rawBody = _interopRequireDefault(require("raw-body"));

var _url = _interopRequireDefault(require("url"));

var _utils = require("./utils");

// export function handle(i18next: I18next, options: HandleOptions = {}): connect.HandleFunction {
function handle(i18next) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return function i18nextMiddleware(req, res, // next: NextFunction
  next) {
    if (_lodash["default"].isFunction(options.ignoreRoutes)) {
      if (options.ignoreRoutes(req, res, options, i18next)) {
        return next();
      }
    } else {
      var ignores = _lodash["default"].isArray(options.ignoreRoutes) && options.ignoreRoutes || [];

      for (var i = 0; i < ignores.length; i++) {
        var urlParts = _url["default"].parse(req.url);

        if (urlParts.path.indexOf(ignores[i]) > -1) {
          return next();
        }
      } // const urlParts: UrlWithStringQuery = URL.parse(req.url);
      // ignores
      //   .filter(ignore => urlParts.path.indexOf(ignore) > -1)
      //   .map(_ignore => {
      //     return next();
      //   });

    }

    var i18n = i18next.cloneInstance({
      initImmediate: false
    });
    i18n.on('languageChanged', function (lng) {
      // Keep language in sync
      req.language = req.locale = req.lng = lng;

      if (res.locals) {
        res.locals.language = lng;
        res.locals.languageDir = i18next.dir(lng);
      }

      if (!res.headersSent) {
        res.setHeader('Content-Language', lng);
      }

      req.languages = i18next.services.languageUtils.toResolveHierarchy(lng);

      if (i18next.services.languageDetector) {
        i18next.services.languageDetector.cacheUserLanguage(req, res, lng);
      }
    });
    var lng = req.lng;
    if (!req.lng && i18next.services.languageDetector) lng = i18next.services.languageDetector.detect(req, res); // set locale

    req.language = req.locale = req.lng = lng;

    if (!res.headersSent) {
      res.setHeader('Content-Language', lng);
    }

    req.languages = i18next.services.languageUtils.toResolveHierarchy(lng); // trigger sync to instance - might trigger async load!

    i18n.changeLanguage(lng || i18next.options.fallbackLng[0]);

    if (req.i18nextLookupName === 'path' && options.removeLngFromUrl) {
      req.url = (0, _utils.removeLngFromUrl)(req.url, i18next.services.languageDetector.options.lookupFromPathIndex);
    }

    var t = i18n.t.bind(i18n);
    var exists = i18n.exists.bind(i18n); // assert for req

    req.i18n = i18n;
    req.t = t; // assert for res -> template

    if (res.locals) {
      res.locals.t = t;
      res.locals.exists = exists;
      res.locals.i18n = i18n;
      res.locals.language = lng;
      res.locals.languageDir = i18next.dir(lng);
    }

    if (i18next.services.languageDetector) i18next.services.languageDetector.cacheUserLanguage(req, res, lng); // load resources

    if (!req.lng) return next();
    i18next.loadLanguages(req.lng, function () {
      return next();
    });
  };
}

function getResourcesHandler(i18next, options) {
  options = options || {};
  var maxAge = options.maxAge || 60 * 60 * 24 * 30;
  return function (req, res) {
    if (!i18next.services.backendConnector) {
      return res.writeHead(404).end(JSON.stringify('i18next-node-middleware-test:: no backend configured'));
    }

    var resources = {};
    res.setHeader('Content-Type', 'application/json;charset=utf-8');

    if (options.cache !== undefined ? options.cache : process.env.NODE_ENV === 'production') {
      res.setHeader('Cache-Control', 'public, max-age=' + maxAge);
      res.setHeader('Expires', new Date(new Date().getTime() + maxAge * 1000).toUTCString());
    } else {
      res.setHeader('Pragma', 'no-cache');
      res.setHeader('Cache-Control', 'no-cache');
    }

    var urlParts = _url["default"].parse(req.url, true);

    var query = urlParts.query;
    var languages = query[options.lngParam || 'lng'] ? query[options.lngParam || 'lng'].split(' ') : [];
    var namespaces = query[options.nsParam || 'ns'] ? query[options.nsParam || 'ns'].split(' ') : []; // extend ns

    namespaces.forEach(function (ns) {
      // if (i18next.options.ns && i18next.options.ns.indexOf(ns) < 0) i18next.options.ns.push(ns);
      if (i18next.options.ns && i18next.options.ns.indexOf(ns) < 0) {
        if (_lodash["default"].isArray(i18next.options.ns)) {
          i18next.options.ns.push(ns);
        }
      }
    });
    i18next.services.backendConnector.load(languages, namespaces, function () {
      languages.forEach(function (lng) {
        namespaces.forEach(function (ns) {
          (0, _utils.setPath)(resources, [lng, ns], i18next.getResourceBundle(lng, ns));
        });
      });
      res.end(JSON.stringify(resources));
    });
  };
}

var missingKeyHandler = function missingKeyHandler(i18next, options) {
  options = options || {}; // return async (req: http.IncomingMessage, res: http.ServerResponse) => {

  return function (req, res) {
    var url = new _url["default"].URL(req.url);
    var params = url.searchParams;
    var lng = params[options.lngParam || 'lng'];
    var ns = params[options.nsParam || 'ns'];

    if (!i18next.services.backendConnector) {
      return res.writeHead(400).end(JSON.stringify('i18next-node-middleware-test:: no backend configured'));
    }

    (0, _rawBody["default"])(req).then(function (buf) {
      i18next.services.backendConnector.saveMissing([lng], ns, buf.length, buf);
      res.end(JSON.stringify('ok'));
    })["catch"](function (err) {
      res.statusCode = err.statusCode;
      res.end(err.message);
    }); // let buffer: Buffer;
    // try {
    //   buffer = await getRawBody(req);
    // } catch (error) {
    //   console.log(JSON.stringify({ error: error.toString(), stack: error.stack }));
    // }
    // i18next.services.backendConnector.saveMissing([lng], ns, buffer.length, buffer);
    // res.end(JSON.stringify('ok'));
  };
};

exports.missingKeyHandler = missingKeyHandler;