/* eslint-disable @typescript-eslint/indent */
import _ from 'lodash';
import { defaults } from '../../common/utils';
import { cookie, header, path, querystring } from '../lookup'; // import {
//   LanguageDetectorAllOptions,
//   LanguageDetectorCaches,
//   LanguageDetectorOptions,
//   LanguageDetectorOrder,
//   LanguageDetectorServices,
// } from '../../../typings';
// import sessionLookup from './languageLookups/session';

function getDefaults() {
  return {
    order: ['path', 'querystring', 'cookie', 'header'],
    lookupQuerystring: 'lng',
    lookupCookie: 'i18next',
    lookupSession: 'lng',
    lookupFromPathIndex: 0,
    // cache user language
    caches: false // ['cookie']
    //cookieExpirationDate: new Date(),
    //cookieDomain: 'myDomain'

  };
}

export class LanguageDetector {
  constructor(services, options, allOptions) {
    this.detectors = {};
    this.init(services, options, allOptions);
  }

  init(services, options = {}, allOptions = {}) {
    this.services = services;
    this.options = defaults(options || getDefaults());
    this.allOptions = allOptions;
    this.addDetector(cookie);
    this.addDetector(querystring);
    this.addDetector(path);
    this.addDetector(header);
  }

  addDetector(detector) {
    this.detectors[detector.name] = detector;
  }

  detect(req, res, detectionOrder) {
    if (!detectionOrder) detectionOrder = this.options.order; // let found;

    return detectionOrder.forEach(detectorName => {
      // if (found || !this.detectors[detectorName]) return;
      if (!this.detectors[detectorName]) return;
      const detector = this.detectors[detectorName];
      let detections = detector.lookup(req, res, this.options);
      if (!detections) return;
      if (!_.isArray(detections)) detections = [detections];
      const selectedLanguages = detections.forEach(lng => {
        if (!_.isString(lng)) return;
        let cleanedLng = this.services.languageUtils.formatLanguageCode(lng);

        if (this.services.languageUtils.isWhitelisted(cleanedLng)) {
          req.headers['i18nextLookupName'] = detectorName;
          return cleanedLng;
        }
      });

      if (_.isArray(selectedLanguages) && selectedLanguages.length > 1) {
        return selectedLanguages[0];
      }
    });
  } // cacheUserLanguage(
  //   req: http.IncomingMessage,
  //   res: http.ServerResponse,
  //   lng: string,
  //   caches?: LanguageDetectorCaches
  // ) {
  //   if (!caches) {
  //     caches = this.options.caches;
  //   }
  //   if (caches) {
  //     caches.forEach(cacheName => {
  //       if (this.detectors[cacheName] && this.detectors[cacheName].cacheUserLanguage)
  //         this.detectors[cacheName].cacheUserLanguage(req, res, lng, this.options);
  //     });
  //   }
  // }


}